package ${package}.config;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Properties;

import org.apache.log4j.Logger;

public final class Settings extends Properties {

	private static final long serialVersionUID = -8954439607147639044L;

	private static final Logger logger = Logger.getLogger(Settings.class);

	private static Settings instance = null;;

	private static String propertiesFile = null;

	protected static String getPropertiesFile() {
		return null != propertiesFile ? propertiesFile : Constants.DEFAULT_PROPERTIES_FILE;
	}

	public static void configure(String properties) {
		propertiesFile = properties;
		instance = null;
	}

	private Settings() {
		try {
			load(Settings.class.getResourceAsStream(getPropertiesFile()));
		} catch (IOException e) {
			Logger.getLogger(getClass()).error(e, e);
		}
	}

	public static Settings getInstance() {
		if (null == instance) {
			instance = new Settings();
		}
		return instance;
	}

	@Override
	public synchronized Object setProperty(String key, String value) {
		Object result = super.setProperty(key, value);
		try {
			saveProperties();
			return result;
		} catch (Exception e) {
			logger.error("Error saving settings: ", e);
		}
		return null;
	}

	private synchronized void saveProperties() throws URISyntaxException,
			IOException {
		URL url = Settings.class.getResource(getPropertiesFile());
		if (url != null) {
			OutputStream os = new FileOutputStream(new File(url.toURI()));
			if (os != null) {
				store(os, "");
				os.flush();
				os.close();
			}
		}
		logger.debug("Settings was successfully written");
	}

	public boolean getBoolean(String key) {
		return Boolean.parseBoolean(getProperty(key));
	}

	public void setBoolean(String key, boolean value) {
		setProperty(key, Boolean.toString(value));
	}

	public int getInteger(String key) {
		try {
			return Integer.parseInt(getProperty(key));
		} catch (Exception e) {
			return 0;
		}
	}

	public void setInteger(String key, int value) {
		setProperty(key, Integer.toString(value));
	}

}
