package ${package}.db;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;

import ${package}.dao.ibatis.session.ConnectionFactory;

import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.session.SqlSession;
import org.apache.log4j.Logger;

public class SQLUtils {

	private static Logger logger = Logger.getLogger(SQLUtils.class);

	public static boolean executeSQLScript(InputStream is) {
		SqlSession session = ConnectionFactory.getSqlSessionFactory().openSession();
		try {
			executeBatchSql(session.getConnection(), new BufferedReader(new InputStreamReader(is)));
			return true;
		} catch (SQLException e) {
			logger.error(e, e);
		} catch (IOException e) {
			logger.error(e, e);
		} finally {
			if (null != session) {
				session.close();
			}
		}
		return false;
	}

	private static void executeBatchSql(Connection connection, BufferedReader reader) throws SQLException, IOException {
		Statement st = connection.createStatement();
		StringBuilder sql = new StringBuilder();
		String line = null;
		String delimiter = ";";
		while (null != (line = reader.readLine())) {
			if (line.trim().toLowerCase().startsWith("delimiter")) {
				delimiter = line.trim().toLowerCase().replace("delimiter", "").trim();
				continue;
			}
			String[] statements = line.split(delimiter);
			if (1 == statements.length) {
				if (!StringUtils.isEmpty(statements[0].replaceAll("[\n\t ]*", ""))) {
					if (line.trim().toLowerCase().endsWith(delimiter)) {
						sql.append(statements[0] + " ");
						st.addBatch(sql.toString());
						sql = new StringBuilder();
					} else {
						sql.append(statements[0] + "\n");
					}
				}
			} else {
				for (int i = 0; i < statements.length; i++) {
					sql.append(statements[i]);
					if (i != statements.length - 1 && !StringUtils.isEmpty(statements[i].replaceAll("[\n\t ]*", ""))) {
						st.addBatch(sql.toString());
						sql = new StringBuilder();
					}
				}
			}
		}
		st.executeBatch();
	}
}
