package ${package}.db;

import ${package}.config.Settings;
import ${package}.dao.ibatis.mappers.UtilsMapper;
import ${package}.dao.ibatis.session.ConnectionFactory;
import ${package}.dao.ibatis.session.SqlSession;

import org.apache.log4j.Logger;

public class SchemaManager {

	private static SchemaManager instance = new SchemaManager();

	private static Logger logger = Logger.getLogger(SchemaManager.class);

	public static SchemaManager getInstance() {
		return instance;
	}

	private long getNewSchemaVersion() {
		return 1;
	}

	public boolean initSchema() {
		boolean result = true;
		try (SqlSession session = ConnectionFactory.getSqlSessionFactory().openSession()) {
			Long version = 0L;
			try {
				version = session.getMapper(UtilsMapper.class).selectSchemaVersion();
			} catch (Exception e) {
				// Do nothing
				logger.warn("Schema doesn't exist");
			}
			if (version == 0) {
				logger.info("Initializing schema...");
				result = createSchema();
				if (result) {
					logger.info("Schema created successfully");
				} else {
					throw new RuntimeException("Schema creation failed");
				}
			} else if (version < getNewSchemaVersion()) {
				logger.info("Initialized schema update");
				result = updateSchema(getNewSchemaVersion());
				if (result) {
					logger.info("Schema updated successfilly");
				} else {
					throw new RuntimeException("Schema update failed");
				}
			}
		}
		return result;
	}

	private boolean updateSchema(long newSchemaVersion) {
		throw new RuntimeException("Not implemented yet");
	}

	private boolean createSchema() {
		return SQLUtils.executeSQLScript(SchemaManager.class.getResourceAsStream(
				Settings.getInstance().getProperty("db.script.name")));
	}
}
